@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -51,6 +51,7 @@ package com.openexchange.imap;
 
 import java.io.IOException;
 import java.io.OutputStream;
+import java.io.OutputStreamWriter;
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.Comparator;
@@ -61,6 +62,7 @@ import javax.mail.internet.InternetAddress;
 import org.apache.commons.logging.Log;
 import com.openexchange.imap.IMAPTracer.TracerState;
 import com.openexchange.imap.config.IMAPConfig;
+import com.openexchange.imap.util.ImapUtility;
 import com.openexchange.mail.MailException;
 import com.openexchange.mail.dataobjects.IDMailMessage;
 import com.openexchange.mail.dataobjects.MailMessage;
@@ -86,12 +88,20 @@ import com.sun.mail.imap.protocol.UID;
 
 /**
  * {@link AllFetch} - Utility class to fetch all messages from a certain IMAP folder.
- * 
+ *
  * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
  */
 public final class AllFetch {
 
-    private static final org.apache.commons.logging.Log LOG = org.apache.commons.logging.LogFactory.getLog(AllFetch.class);
+    /**
+     * The logger constant.
+     */
+    protected static final org.apache.commons.logging.Log LOG = com.openexchange.log.Log.valueOf(org.apache.commons.logging.LogFactory.getLog(AllFetch.class));
+
+    /**
+     * Whether debug logging is enabled.
+     */
+    protected static final boolean DEBUG = LOG.isDebugEnabled();
 
     /**
      * Initializes a new {@link AllFetch}.
@@ -104,7 +114,7 @@ public final class AllFetch {
 
         /**
          * Handles given <code>com.sun.mail.imap.protocol.Item</code> instance and applies it to given message.
-         * 
+         *
          * @param item The item to handle
          * @param msg The message to apply to
          * @param logger The logger
@@ -199,37 +209,7 @@ public final class AllFetch {
                 // In-Reply-To and Message-Id
                 m.addHeader("In-Reply-To", envelope.inReplyTo);
                 m.addHeader("Message-Id", envelope.messageId);
-                // Subject
-                /*-
-                 * Hmm... Why does ENVELOPE FETCH response omit CR?LFs in subject?!
-                 * 
-                 * Example:
-                 * Subject: =?UTF-8?Q?Nur_noch_kurze_Zeit:_1_Freimona?=
-                 *  =?UTF-8?Q?t_f=C3=BCr_3_erfolgreiche_Einladungen?=
-                 *  
-                 * is transferred as:
-                 * =?UTF-8?Q?Nur_noch_kurze_Zeit:_1_Freimona?= =?UTF-8?Q?t_f=C3=BCr_3_erfolgreiche_Einladungen?=
-                 */
-                final String subject;
-                if (null == envelope.subject) {
-                    subject = "";
-                } else {
-                    final char[] chars = envelope.subject.toCharArray();
-                    final StringBuilder sb = new StringBuilder(chars.length);
-                    int i = 0;
-                    while (i < chars.length) {
-                        final char c = chars[i];
-                        if ('\t' != c && '\r' != c && '\n' != c) {
-                            if (' ' == c && (i + 1) < chars.length && ' ' == chars[i + 1]) {
-                                i++;
-                            }
-                            sb.append(c);
-                        }
-                        i++;
-                    }
-                    subject = MIMEMessageUtility.decodeEnvelopeHeader(sb.toString());
-                }
-                m.setSubject(subject);
+                m.setSubject(MIMEMessageUtility.decodeEnvelopeSubject(envelope.subject));
             }
 
             private String addrs2String(final InternetAddress[] addrs) {
@@ -260,7 +240,7 @@ public final class AllFetch {
 
         /**
          * Gets the Fetch item string.
-         * 
+         *
          * @return The Fetch item string
          */
         public String getItemString() {
@@ -269,7 +249,7 @@ public final class AllFetch {
 
         /**
          * Gets the item class.
-         * 
+         *
          * @return The item class
          */
         public Class<? extends Item> getItemClass() {
@@ -278,7 +258,7 @@ public final class AllFetch {
 
         /**
          * Gets the item handler.
-         * 
+         *
          * @return The item handler
          */
         public FetchItemHandler getItemHandler() {
@@ -295,7 +275,7 @@ public final class AllFetch {
 
     /**
      * Fetches all messages from given IMAP folder and pre-fills instances with UID, folder fullname and received date.
-     * 
+     *
      * @param imapFolder The IMAP folder
      * @param ascending <code>true</code> to order messages by received date in ascending order; otherwise descending
      * @param config The IMAP configuration
@@ -311,7 +291,7 @@ public final class AllFetch {
      * Fetches all messages from given IMAP folder and pre-fills instances with fullname and given low-cost fetch item list.
      * <p>
      * Since returned instances are sorted, the low-cost fetch item list must contain <code>"INTERNALDATE"</code>.
-     * 
+     *
      * @param imapFolder The IMAP folder
      * @param items The low-cost fetch items
      * @param ascending <code>true</code> to order messages by received date in ascending order; otherwise descending
@@ -321,22 +301,22 @@ public final class AllFetch {
      * @throws MessagingException If an error occurs in underlying protocol
      */
     public static MailMessage[] fetchLowCost(final IMAPFolder imapFolder, final LowCostItem[] items, final boolean ascending, final IMAPConfig config, final Session session) throws MessagingException {
-        if (imapFolder.getMessageCount() == 0) {
+        final int messageCount = imapFolder.getMessageCount();
+        if (messageCount <= 0) {
             /*
              * Empty folder...
              */
             return new MailMessage[0];
         }
-        final org.apache.commons.logging.Log logger = LOG;
         return (MailMessage[]) (imapFolder.doCommand(new IMAPFolder.ProtocolCommand() {
 
-            public Object doCommand(final IMAPProtocol p) throws ProtocolException {
+            public Object doCommand(final IMAPProtocol protocol) throws ProtocolException {
                 /*-
                  * Arguments:  sequence set
                  * message data item names or macro
-                 * 
+                 *
                  * Responses:  untagged responses: FETCH
-                 * 
+                 *
                  * Result:     OK - fetch completed
                  *             NO - fetch error: can't fetch that data
                  *             BAD - command unknown or arguments invalid
@@ -345,30 +325,21 @@ public final class AllFetch {
                 {
                     final String lowCostItems = getFetchCommand(items);
                     command =
-                        new StringBuilder(12 + lowCostItems.length()).append("FETCH 1:* (").append(lowCostItems).append(')').toString();
+                        new StringBuilder(12 + lowCostItems.length()).append("FETCH ").append(1 == messageCount ? "1" : "1:*").append(" (").append(
+                            lowCostItems).append(')').toString();
                 }
                 /*
                  * Enable tracer
                  */
-                final SBOutputStream sbout = new SBOutputStream();
-                TracerState tracerState = null;
+                final SBOutputStream sbout = DEBUG ? new SBOutputStream() : null;
+                final TracerState tracerState = DEBUG ? traceStateFor(protocol, sbout) : null;
                 try {
-                    tracerState = IMAPTracer.enableTrace(p, sbout);
-                } catch (final SecurityException e) {
-                    logger.error(e.getMessage(), e);
-                } catch (final IllegalArgumentException e) {
-                    logger.error(e.getMessage(), e);
-                } catch (final NoSuchFieldException e) {
-                    logger.error(e.getMessage(), e);
-                } catch (final IllegalAccessException e) {
-                    logger.error(e.getMessage(), e);
-                }
-                try {
-                    final Response[] r = p.command(command, null);
+                    final Response[] r = protocol.command(command, null);
                     final int len = r.length - 1;
                     final Response response = r[len];
                     final List<MailMessage> l = new ArrayList<MailMessage>(len);
                     if (response.isOK()) {
+                        // final int recentCount = imapFolder.getNewMessageCount();
                         final String fullname = imapFolder.getFullName();
                         for (int j = 0; j < len; j++) {
                             final Response resp = r[j];
@@ -376,13 +347,14 @@ public final class AllFetch {
                                 final FetchResponse fr = (FetchResponse) resp;
                                 try {
                                     final MailMessage m = new IDMailMessage(null, fullname);
+                                    // m.setRecentCount(recentCount);
                                     for (final LowCostItem lowCostItem : items) {
                                         final Item item =
                                             getItemOf(lowCostItem.getItemClass(), fr, lowCostItem.getItemString(), config, session);
                                         try {
-                                            lowCostItem.getItemHandler().handleItem(item, m, logger);
+                                            lowCostItem.getItemHandler().handleItem(item, m, LOG);
                                         } catch (final MailException e) {
-                                            logger.error(e.getMessage(), e);
+                                            LOG.error(e.getMessage(), e);
                                         }
                                     }
                                     l.add(m);
@@ -392,54 +364,124 @@ public final class AllFetch {
                                         sb.insert(0, "\nIMAP trace:\n");
                                         sb.insert(0, e.getMessage());
                                         sb.insert(0, "Detected invalid FETCH response which will be ignored. Error:\n");
-                                        logger.warn(sb.toString(), e);
+                                        LOG.warn(sb.toString(), e);
                                     } else {
-                                        logger.warn(e.getMessage(), e);
+                                        LOG.warn(e.getMessage(), e);
                                     }
                                 }
                                 r[j] = null;
                             }
                         }
-                        p.notifyResponseHandlers(r);
+                        protocol.notifyResponseHandlers(r);
                     } else if (response.isBAD()) {
+                        if (ImapUtility.isInvalidMessageset(response)) {
+                            return new MailMessage[0];
+                        }
                         throw new BadCommandException(IMAPException.getFormattedMessage(
                             IMAPException.Code.PROTOCOL_ERROR,
                             command,
                             response.toString()));
                     } else if (response.isNO()) {
+                        /*
+                         * Check number of messages
+                         */
+                        try {
+                            if (IMAPCommandsCollection.getTotal(imapFolder) <= 0) {
+                                return new MailMessage[0];
+                            }
+                        } catch (final MessagingException e) {
+                            LOG.warn("STATUS command failed. Throwing original exception: " + response.toString(), e);
+                        }
                         throw new CommandFailedException(IMAPException.getFormattedMessage(
                             IMAPException.Code.PROTOCOL_ERROR,
                             command,
                             response.toString()));
                     } else {
-                        p.handleResult(response);
+                        protocol.handleResult(response);
                     }
                     Collections.sort(l, ascending ? ASC_COMP : DESC_COMP);
                     return l.toArray(new MailMessage[l.size()]);
+                    // } catch (final MessagingException e) {
+                    // throw new ProtocolException(e.getMessage(), e);
                 } finally {
-                    if (null != tracerState) {
-                        try {
-                            IMAPTracer.restoreTraceState(p, tracerState);
-                        } catch (final SecurityException e) {
-                            logger.error(e.getMessage(), e);
-                        } catch (final IllegalArgumentException e) {
-                            logger.error(e.getMessage(), e);
-                        } catch (final NoSuchFieldException e) {
-                            logger.error(e.getMessage(), e);
-                        } catch (final IllegalAccessException e) {
-                            logger.error(e.getMessage(), e);
-                        }
+                    if (DEBUG) {
+                        restoreTracerFor(protocol, sbout, tracerState);
                     }
                 }
             }
-
         }));
     }
 
+    /**
+     * Restores trace state for specified IMAP protocol.
+     *
+     * @param protocol The protocol whose trace state shall be restored
+     * @param sbout The output stream written to in the meantime
+     * @param tracerState The trace state
+     */
+    protected static void restoreTracerFor(final IMAPProtocol protocol, final SBOutputStream sbout, final TracerState tracerState) {
+        if (null == tracerState) {
+            return;
+        }
+        try {
+            IMAPTracer.restoreTraceState(protocol, tracerState);
+            if (tracerState.isTrace()) {
+                /*
+                 * Trace was enabled before, thus write trace to previous output stream to maintain debug logs properly.
+                 */
+                final StringBuilder sb = sbout.getTrace();
+                try {
+                    /*
+                     * DON'T CLOSE THE WRITER BECAUSE IT CLOSES UNDERLYING STREAM, TOO!!!
+                     */
+                    final OutputStreamWriter writer = new OutputStreamWriter(tracerState.getOut(), "UTF-8");
+                    writer.write(sb.toString());
+                    writer.flush();
+                } catch (final IOException e) {
+                    // Writing trace to stream failed...
+                }
+            }
+        } catch (final SecurityException e) {
+            LOG.error(e.getMessage(), e);
+        } catch (final IllegalArgumentException e) {
+            LOG.error(e.getMessage(), e);
+        } catch (final NoSuchFieldException e) {
+            LOG.error(e.getMessage(), e);
+        } catch (final IllegalAccessException e) {
+            LOG.error(e.getMessage(), e);
+        } catch (final RuntimeException e) {
+            LOG.error(e.getMessage(), e);
+        }
+    }
+
+    /**
+     * Gets the trace state for specified IMAP protocol.
+     *
+     * @param protocol The protocol whose trace state shall be returned
+     * @param sbout The output stream to write to
+     * @return The trace state
+     */
+    protected static TracerState traceStateFor(final IMAPProtocol protocol, final SBOutputStream sbout) {
+        try {
+            return IMAPTracer.enableTrace(protocol, sbout);
+        } catch (final SecurityException e) {
+            LOG.error(e.getMessage(), e);
+        } catch (final IllegalArgumentException e) {
+            LOG.error(e.getMessage(), e);
+        } catch (final NoSuchFieldException e) {
+            LOG.error(e.getMessage(), e);
+        } catch (final IllegalAccessException e) {
+            LOG.error(e.getMessage(), e);
+        } catch (final RuntimeException e) {
+            LOG.error(e.getMessage(), e);
+        }
+        return null;
+    }
+
     /**
      * A {@link Comparator} comparing instances of {@link MailMessage} by their received date in ascending order.
      */
-    static final Comparator<MailMessage> ASC_COMP = new Comparator<MailMessage>() {
+    protected static final Comparator<MailMessage> ASC_COMP = new Comparator<MailMessage>() {
 
         public int compare(final MailMessage m1, final MailMessage m2) {
             final Date d1 = m1.getReceivedDate();
@@ -452,7 +494,7 @@ public final class AllFetch {
     /**
      * A {@link Comparator} comparing instances of {@link MailMessage} by their received date in descending order.
      */
-    static final Comparator<MailMessage> DESC_COMP = new Comparator<MailMessage>() {
+    protected static final Comparator<MailMessage> DESC_COMP = new Comparator<MailMessage>() {
 
         public int compare(final MailMessage m1, final MailMessage m2) {
             final Date d1 = m1.getReceivedDateDirect();
@@ -464,18 +506,18 @@ public final class AllFetch {
 
     /**
      * Compares given object references being <code>null</code>.
-     * 
+     *
      * @param o1 The first object reference
      * @param o2 The second object reference
      * @return An {@link Integer} of <code>-1</code> if first reference is <code>null</code> but the second is not, an {@link Integer} of
      *         <code>1</code> if first reference is not <code>null</code> but the second is, an {@link Integer} of <code>0</code> if both
      *         references are <code>null</code>, or returns <code>null</code> if both references are not <code>null</code>
      */
-    static Integer compareReferences(final Object o1, final Object o2) {
+    protected static Integer compareReferences(final Object o1, final Object o2) {
         if ((o1 == null)) {
-            return (o2 == null) ? /*both null*/Integer.valueOf(0) : Integer.valueOf(-1);
+            return (o2 == null) ? /* both null */Integer.valueOf(0) : Integer.valueOf(-1);
         }
-        return (o2 == null) ? Integer.valueOf(1) : /*both not null*/null;
+        return (o2 == null) ? Integer.valueOf(1) : /* both not null */null;
     }
 
     /**
@@ -516,7 +558,7 @@ public final class AllFetch {
 
         /**
          * Gets the trace.
-         * 
+         *
          * @return The trace
          */
         public StringBuilder getTrace() {
@@ -528,7 +570,7 @@ public final class AllFetch {
     /**
      * Gets the item associated with given class in specified <i>FETCH</i> response; throws an appropriate protocol exception if not present
      * in given <i>FETCH</i> response.
-     * 
+     *
      * @param <I> The returned item's class
      * @param clazz The item class to look for
      * @param fetchResponse The <i>FETCH</i> response
@@ -547,7 +589,7 @@ public final class AllFetch {
 
     /**
      * Gets the item associated with given class in specified <i>FETCH</i> response.
-     * 
+     *
      * @param <I> The returned item's class
      * @param clazz The item class to look for
      * @param fetchResponse The <i>FETCH</i> response
@@ -568,7 +610,7 @@ public final class AllFetch {
     /**
      * Generates a new protocol exception according to following template:<br>
      * <code>&quot;Missing &lt;itemName&gt; item in FETCH response.&quot;</code>
-     * 
+     *
      * @param itemName The item name; e.g. <code>UID</code>, <code>FLAGS</code>, etc.
      * @param config The IMAP configuration
      * @param session The session
@@ -583,7 +625,7 @@ public final class AllFetch {
 
     /**
      * Gets the fetch items' string representation; e.g <code>"UID INTERNALDATE"</code>.
-     * 
+     *
      * @param items The items
      * @return The string representation
      */
