@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -51,19 +51,27 @@ package com.openexchange.imap;
 
 import static com.openexchange.mail.dataobjects.MailFolder.DEFAULT_FOLDER_ID;
 import java.util.Set;
+import java.util.concurrent.ConcurrentHashMap;
+import java.util.concurrent.ConcurrentMap;
 import javax.mail.Flags;
 import javax.mail.Folder;
 import javax.mail.MessagingException;
+import com.openexchange.config.ConfigurationService;
 import com.openexchange.groupware.contexts.Context;
 import com.openexchange.groupware.contexts.impl.ContextException;
 import com.openexchange.groupware.contexts.impl.ContextStorage;
 import com.openexchange.imap.acl.ACLExtension;
-import com.openexchange.imap.acl.ACLExtensionFactory;
+import com.openexchange.imap.cache.ListLsubCache;
+import com.openexchange.imap.cache.ListLsubEntry;
+import com.openexchange.imap.cache.ListLsubRuntimeException;
 import com.openexchange.imap.cache.RightsCache;
 import com.openexchange.imap.config.IMAPConfig;
+import com.openexchange.imap.notify.internal.IMAPNotifierMessageRecentListener;
+import com.openexchange.imap.services.IMAPServiceRegistry;
 import com.openexchange.mail.MailException;
 import com.openexchange.mail.api.MailMessageStorage;
 import com.openexchange.mail.api.enhanced.MailMessageStorageLong;
+import com.openexchange.mail.dataobjects.MailFolder;
 import com.openexchange.mail.mime.MIMEMailException;
 import com.openexchange.mail.mime.MIMESessionPropertyNames;
 import com.openexchange.mail.usersetting.UserSettingMail;
@@ -77,12 +85,12 @@ import com.sun.mail.imap.Rights.Right;
 /**
  * {@link IMAPFolderWorker} - An abstract class that extends {@link MailMessageStorage} by convenience methods for working on a certain IMAP
  * folder.
- * 
+ *
  * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
  */
 public abstract class IMAPFolderWorker extends MailMessageStorageLong {
 
-    private static final org.apache.commons.logging.Log LOG = org.apache.commons.logging.LogFactory.getLog(IMAPFolderWorker.class);
+    private static final org.apache.commons.logging.Log LOG = com.openexchange.log.Log.valueOf(org.apache.commons.logging.LogFactory.getLog(IMAPFolderWorker.class));
 
     protected static final String STR_INBOX = "INBOX";
 
@@ -90,10 +98,59 @@ public abstract class IMAPFolderWorker extends MailMessageStorageLong {
 
     protected static final Flags FLAGS_SEEN = new Flags(Flags.Flag.SEEN);
 
+    private static final class FailFastError {
+
+        final MessagingException error;
+        final long stamp;
+
+        FailFastError(final MessagingException e) {
+            super();
+            this.error = e;
+            this.stamp = System.currentTimeMillis();
+        }
+    }
+
+    private static final ConcurrentMap<String, FailFastError> FAIL_FAST = new ConcurrentHashMap<String, FailFastError>();
+
+    private static volatile Integer failFastTimeout;
+    private static int failFastTimeout() {
+        Integer tmp = failFastTimeout;
+        if (null == tmp) {
+            synchronized (IMAPFolderWorker.class) {
+                tmp = failFastTimeout;
+                if (null == tmp) {
+                    final ConfigurationService service = IMAPServiceRegistry.getService(ConfigurationService.class);
+                    tmp = Integer.valueOf(null == service ? 10000 : service.getIntProperty("com.openexchange.imap.failFastTimeout", 10000));
+                    failFastTimeout = tmp;
+                }
+            }
+        }
+        return tmp.intValue();
+    }
+
+    /**
+     * Checks for possible fail-fast error.
+     * 
+     * @param imapStore The IMAP store to check
+     * @param fullName The mailbox full name
+     * @throws MessagingException If such a fail-fast error exists and has not elapsed, yet
+     */
+    public static void checkFailFast(final IMAPStore imapStore, final String fullName) throws MessagingException {
+        final String key = fullName + '@' + imapStore.toString();
+        final FailFastError failFastError = FAIL_FAST.get(key);
+        if (null == failFastError) {
+            return;
+        }
+        if ((System.currentTimeMillis() - failFastError.stamp) <= failFastTimeout()) {
+            throw failFastError.error;
+        }
+        FAIL_FAST.remove(key, failFastError);
+    }
+
     /*
      * Fields
      */
-    protected final IMAPStore imapStore;
+    protected final AccessedIMAPStore imapStore;
 
     protected final Session session;
 
@@ -115,17 +172,17 @@ public abstract class IMAPFolderWorker extends MailMessageStorageLong {
 
     /**
      * Initializes a new {@link IMAPFolderWorker}.
-     * 
+     *
      * @param imapStore The IMAP store
      * @param imapAccess The IMAP access
      * @param session The session providing needed user data
      * @throws IMAPException If context lading fails
      */
-    public IMAPFolderWorker(final IMAPStore imapStore, final IMAPAccess imapAccess, final Session session) throws IMAPException {
+    public IMAPFolderWorker(final AccessedIMAPStore imapStore, final IMAPAccess imapAccess, final Session session) throws IMAPException {
         super();
         this.imapStore = imapStore;
         this.imapAccess = imapAccess;
-        this.accountId = imapAccess.getAccountId();
+        accountId = imapAccess.getAccountId();
         this.session = session;
         try {
             ctx = ContextStorage.getStorageContext(session.getContextId());
@@ -134,24 +191,33 @@ public abstract class IMAPFolderWorker extends MailMessageStorageLong {
         }
         usm = UserSettingMailStorage.getInstance().getUserSettingMail(session.getUserId(), ctx);
         imapConfig = imapAccess.getIMAPConfig();
-        aclExtension = ACLExtensionFactory.getInstance().getACLExtension(imapConfig);
+        aclExtension = imapConfig.getACLExtension();
+    }
+
+    private void openFolder(final int desiredMode, final IMAPFolder imapFolder) throws MessagingException {
+        try {
+            imapFolder.open(desiredMode);
+        } catch (final MessagingException e) {
+            if (toUpperCase(e.getMessage()).indexOf("[INUSE]") >= 0) {
+                FAIL_FAST.put(imapFolder.getFullName() + '@' + imapStore.toString(), new FailFastError(e));
+            }
+            throw e;
+        }
     }
 
     @Override
     public void releaseResources() throws MailException {
-        if (null != imapFolder) {
-            closeIMAPFolder();
-        }
+        closeIMAPFolder();
     }
 
     /**
-     * Reports a modification of the IMAP folder denoted by specified fullname. If stored IMAP folder's fullname equals specified fullname,
-     * it is closed quietly.
-     * 
-     * @param modifiedFullname The fullname of the folder which has been modified
+     * Reports a modification of the IMAP folder denoted by specified full name. If stored IMAP folder's full name equals specified full
+     * name, it is closed quietly.
+     *
+     * @param modifiedFullName The full name of the folder which has been modified
      */
-    public void notifyIMAPFolderModification(final String modifiedFullname) {
-        if ((null == imapFolder) || !modifiedFullname.equals(imapFolder.getFullName())) {
+    public void notifyIMAPFolderModification(final String modifiedFullName) {
+        if ((null == imapFolder) || !modifiedFullName.equals(imapFolder.getFullName())) {
             /*
              * Modified folder did not affect remembered IMAP folder
              */
@@ -166,13 +232,13 @@ public abstract class IMAPFolderWorker extends MailMessageStorageLong {
     }
 
     /**
-     * Reports a modification of the IMAP folders denoted by specified set of fullnames. If stored IMAP folder's fullname is contained in
-     * set of fullnames, it is closed quietly.
-     * 
-     * @param modifiedFullnames The fullnames of the folders which have been modified
+     * Reports a modification of the IMAP folders denoted by specified set of full names. If stored IMAP folder's full name is contained in
+     * set of full names, it is closed quietly.
+     *
+     * @param modifiedFullNames The full names of the folders which have been modified
      */
-    public void notifyIMAPFolderModification(final Set<String> modifiedFullnames) {
-        if ((null == imapFolder) || !modifiedFullnames.contains(imapFolder.getFullName())) {
+    public void notifyIMAPFolderModification(final Set<String> modifiedFullNames) {
+        if ((null == imapFolder) || !modifiedFullNames.contains(imapFolder.getFullName())) {
             /*
              * Modified folders did not affect remembered IMAP folder
              */
@@ -185,11 +251,18 @@ public abstract class IMAPFolderWorker extends MailMessageStorageLong {
         }
     }
 
+    /**
+     * Closes remembered IMAP folder (if non-<code>null</code>).
+     *
+     * @throws MailException If closing remembered IMAP folder fails
+     */
     private void closeIMAPFolder() throws MailException {
         try {
-            imapFolder.close(false);
+            if (null != imapFolder) {
+                imapFolder.close(false);
+            }
         } catch (final IllegalStateException e) {
-            LOG.warn("Invoked close() on a closed folder", e);
+            LOG.debug("Invoked close() on a closed folder", e);
         } catch (final MessagingException e) {
             throw MIMEMailException.handleMessagingException(e, imapConfig, session);
         } finally {
@@ -208,54 +281,55 @@ public abstract class IMAPFolderWorker extends MailMessageStorageLong {
     /**
      * Determine if field {@link #imapFolder} indicates to hold messages.<br>
      * <b>NOTE</b>: This method assumes that field {@link #imapFolder} is <b>not</b> <code>null</code>.
-     * 
+     *
      * <pre>
      * return ((imapFolder.getType() &amp; IMAPFolder.HOLDS_MESSAGES) == 1)
      * </pre>
-     * 
+     *
      * @return <code>true</code> if field {@link #imapFolder} indicates to hold messages
      * @throws MessagingException If a messaging error occurs
+     * @throws MailException If a messaging error occurs
      */
-    protected boolean holdsMessages() throws MessagingException {
+    protected boolean holdsMessages() throws MailException, MessagingException {
         if (holdsMessages == -1) {
-            holdsMessages = ((imapFolder.getType() & Folder.HOLDS_MESSAGES) == 0) ? 0 : 1;
+            holdsMessages = ListLsubCache.getCachedLISTEntry(imapFolder.getFullName(), accountId, imapFolder, session).canOpen() ? 1 : 0;
         }
         return holdsMessages > 0;
     }
 
     /**
      * Sets and opens (only if exists) the folder in a safe manner, checks if selectable and for right {@link Right#READ}
-     * 
-     * @param fullname The folder fullname
+     *
+     * @param fullName The folder full name
      * @param desiredMode The desired opening mode (either {@link Folder#READ_ONLY} or {@link Folder#READ_WRITE})
      * @return The properly opened IMAP folder
      * @throws MessagingException If a messaging error occurs
      * @throws MailException If user does not hold sufficient rights to open the IMAP folder in desired mode
      */
-    protected final IMAPFolder setAndOpenFolder(final String fullname, final int desiredMode) throws MessagingException, MailException {
-        return setAndOpenFolder(null, fullname, desiredMode);
+    protected final IMAPFolder setAndOpenFolder(final String fullName, final int desiredMode) throws MessagingException, MailException {
+        return setAndOpenFolder(null, fullName, desiredMode);
     }
 
     /**
      * Sets and opens (only if exists) the folder in a safe manner, checks if selectable and for right {@link Right#READ}
-     * 
+     *
      * @param imapFolder The IMAP folder to check against
-     * @param fullname The folder fullname
+     * @param fullName The folder full name
      * @param desiredMode The desired opening mode (either {@link Folder#READ_ONLY} or {@link Folder#READ_WRITE})
      * @return The properly opened IMAP folder
      * @throws MessagingException If a messaging error occurs
      * @throws MailException If user does not hold sufficient rights to open the IMAP folder in desired mode
      */
-    protected final IMAPFolder setAndOpenFolder(final IMAPFolder imapFolder, final String fullname, final int desiredMode) throws MessagingException, MailException {
-        if (null == fullname) {
+    protected final IMAPFolder setAndOpenFolder(final IMAPFolder imapFolder, final String fullName, final int desiredMode) throws MessagingException, MailException {
+        if (null == fullName) {
             throw new MailException(MailException.Code.MISSING_FULLNAME);
         }
-        final boolean isDefaultFolder = DEFAULT_FOLDER_ID.equals(fullname);
+        final boolean isDefaultFolder = DEFAULT_FOLDER_ID.equals(fullName);
         final boolean isIdenticalFolder;
         if (isDefaultFolder) {
             isIdenticalFolder = (imapFolder == null ? false : imapFolder instanceof DefaultFolder);
         } else {
-            isIdenticalFolder = (imapFolder == null ? false : imapFolder.getFullName().equals(fullname));
+            isIdenticalFolder = (imapFolder == null ? false : imapFolder.getFullName().equals(fullName));
         }
         if (imapFolder != null) {
             /*
@@ -280,7 +354,7 @@ public abstract class IMAPFolderWorker extends MailMessageStorageLong {
                      * Folder is open, so close folder
                      */
                     try {
-                        imapFolder.close(false);
+                        imapFolder.close(false/*Folder.READ_WRITE == mode*/);
                     } finally {
                         if (imapFolder == this.imapFolder) {
                             resetIMAPFolder();
@@ -321,55 +395,100 @@ public abstract class IMAPFolderWorker extends MailMessageStorageLong {
                         STR_FALSE)) && IMAPCommandsCollection.isReadOnly(imapFolder)) {
                         throw IMAPException.create(IMAPException.Code.READ_ONLY_FOLDER, imapConfig, session, imapFolderFullname);
                     }
+                    if (imapStore.notifyRecent() && (desiredMode == Folder.READ_WRITE)) {
+                        IMAPNotifierMessageRecentListener.addNotifierFor(imapFolder, fullName, accountId, session, true);
+                    }
                     /*
                      * Open identical folder in right mode
                      */
-                    imapFolder.open(desiredMode);
+                    openFolder(desiredMode, imapFolder);
                     return imapFolder;
                 }
             }
         }
-        final IMAPFolder retval =
-            (isDefaultFolder ? (IMAPFolder) imapStore.getDefaultFolder() : (IMAPFolder) imapStore.getFolder(fullname));
+        final IMAPFolder retval = (isDefaultFolder ? (IMAPFolder) imapStore.getDefaultFolder() : (IMAPFolder) imapStore.getFolder(fullName));
+        if (imapStore.notifyRecent() && (desiredMode == Folder.READ_WRITE)) {
+            IMAPNotifierMessageRecentListener.addNotifierFor(retval, fullName, accountId, session, true);
+        }
         /*
          * Obtain folder lock once to avoid multiple acquire/releases when invoking folder's getXXX() methods
          */
         synchronized (retval) {
-            if (!isDefaultFolder && !retval.exists()) {
-                throw IMAPException.create(IMAPException.Code.FOLDER_NOT_FOUND, imapConfig, session, retval.getFullName());
+            final ListLsubEntry listEntry = ListLsubCache.getCachedLISTEntry(fullName, accountId, retval, session);
+            if (!isDefaultFolder && !STR_INBOX.equals(fullName) && (!listEntry.exists())) {
+                throw IMAPException.create(
+                    IMAPException.Code.FOLDER_NOT_FOUND,
+                    imapConfig,
+                    session,
+                    isDefaultFolder ? MailFolder.DEFAULT_FOLDER_NAME : fullName);
             }
             if ((desiredMode != Folder.READ_ONLY) && (desiredMode != Folder.READ_WRITE)) {
                 throw IMAPException.create(IMAPException.Code.UNKNOWN_FOLDER_MODE, imapConfig, session, Integer.valueOf(desiredMode));
             }
-            final boolean selectable = isSelectable(retval);
+            final boolean selectable = listEntry.canOpen();
             if (!selectable) { // NoSelect
-                throw IMAPException.create(IMAPException.Code.FOLDER_DOES_NOT_HOLD_MESSAGES, imapConfig, session, retval.getFullName());
+                throw IMAPException.create(
+                    IMAPException.Code.FOLDER_DOES_NOT_HOLD_MESSAGES,
+                    imapConfig,
+                    session,
+                    isDefaultFolder ? MailFolder.DEFAULT_FOLDER_NAME : fullName);
             }
             try {
                 if (imapConfig.isSupportsACLs() && !aclExtension.canRead(RightsCache.getCachedRights(retval, true, session, accountId))) {
-                    throw IMAPException.create(IMAPException.Code.NO_FOLDER_OPEN, imapConfig, session, retval.getFullName());
+                    throw IMAPException.create(
+                        IMAPException.Code.NO_FOLDER_OPEN,
+                        imapConfig,
+                        session,
+                        isDefaultFolder ? MailFolder.DEFAULT_FOLDER_NAME : fullName);
                 }
             } catch (final MessagingException e) {
-                throw IMAPException.create(IMAPException.Code.NO_ACCESS, imapConfig, session, e, retval.getFullName());
+                throw IMAPException.create(
+                    IMAPException.Code.NO_ACCESS,
+                    imapConfig,
+                    session,
+                    e,
+                    isDefaultFolder ? MailFolder.DEFAULT_FOLDER_NAME : fullName);
             }
             if ((Folder.READ_WRITE == desiredMode) && (!selectable) && STR_FALSE.equalsIgnoreCase(imapAccess.getMailProperties().getProperty(
                 MIMESessionPropertyNames.PROP_ALLOWREADONLYSELECT,
                 STR_FALSE)) && IMAPCommandsCollection.isReadOnly(retval)) {
-                throw IMAPException.create(IMAPException.Code.READ_ONLY_FOLDER, imapConfig, session, retval.getFullName());
+                throw IMAPException.create(
+                    IMAPException.Code.READ_ONLY_FOLDER,
+                    imapConfig,
+                    session,
+                    isDefaultFolder ? MailFolder.DEFAULT_FOLDER_NAME : fullName);
             }
-            retval.open(desiredMode);
+            openFolder(desiredMode, retval);
         }
         return retval;
     }
 
     /**
-     * Checks if specified IMAP folder is allowed for being selected through SELECT command.
-     * 
-     * @param imapFolder The IMAP folder to check
-     * @return <code>true</code> if specified IMAP folder is allowed for being selected through SELECT command; otherwise <code>false</code>
-     * @throws MessagingException If IMAP folder's type cannot be determined
+     * Handles specified {@link RuntimeException} instance.
+     *
+     * @param e The runtime exception to handle
+     * @return An appropriate {@link MailException}
      */
-    private static final boolean isSelectable(final IMAPFolder imapFolder) throws MessagingException {
-        return (imapFolder.getType() & Folder.HOLDS_MESSAGES) == Folder.HOLDS_MESSAGES;
+    protected MailException handleRuntimeException(final RuntimeException e) {
+        if (e instanceof ListLsubRuntimeException) {
+            ListLsubCache.clearCache(accountId, session);
+            return new MailException(MailException.Code.INTERRUPT_ERROR, e, e.getMessage());
+        }
+        return new MailException(MailException.Code.UNEXPECTED_ERROR, e, e.getMessage());
     }
+
+    /** ASCII-wise upper-case */
+    private static String toUpperCase(final CharSequence chars) {
+        if (null == chars) {
+            return null;
+        }
+        final int length = chars.length();
+        final StringBuilder builder = new StringBuilder(length);
+        for (int i = 0; i < length; i++) {
+            final char c = chars.charAt(i);
+            builder.append((c >= 'a') && (c <= 'z') ? (char) (c & 0x5f) : c);
+        }
+        return builder.toString();
+    }
+
 }
