@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -59,7 +59,7 @@ import com.openexchange.session.Session;
 
 /**
  * {@link IMAPProvider} - The provider for IMAP protocol.
- * 
+ *
  * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
  */
 public final class IMAPProvider extends MailProvider {
@@ -67,13 +67,13 @@ public final class IMAPProvider extends MailProvider {
     /**
      * IMAP protocol.
      */
-    public static final Protocol PROTOCOL_IMAP = new Protocol("imap", "imaps");
+    public static final Protocol PROTOCOL_IMAP = IMAPProtocol.getInstance();
 
     private static final IMAPProvider instance = new IMAPProvider();
 
     /**
      * Gets the singleton instance of IMAP provider.
-     * 
+     *
      * @return The singleton instance of IMAP provider
      */
     public static IMAPProvider getInstance() {
