@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -49,14 +49,14 @@
 
 package com.openexchange.imap.acl;
 
-import java.io.IOException;
+import java.util.Map;
 import java.util.concurrent.atomic.AtomicBoolean;
-import com.openexchange.imap.IMAPException;
 import com.openexchange.imap.config.IMAPConfig;
+import com.openexchange.mailaccount.MailAccount;
 
 /**
  * {@link ACLExtensionFactory} - Factory for ACL extension.
- * 
+ *
  * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
  */
 public final class ACLExtensionFactory {
@@ -73,7 +73,7 @@ public final class ACLExtensionFactory {
 
     /**
      * Gets the factory instance.
-     * 
+     *
      * @return The factory instance.
      */
     public static ACLExtensionFactory getInstance() {
@@ -94,18 +94,27 @@ public final class ACLExtensionFactory {
 
     /**
      * Gets the appropriate ACL extension for the IMAP server denoted by specified IMAP configuration.
-     * 
+     *
      * @param imapConfig The IMAP configuration providing needed access data.
      * @return The appropriate ACL extension
-     * @throws IMAPException If an I/O error occurs
      */
-    public ACLExtension getACLExtension(final IMAPConfig imapConfig) throws IMAPException {
+    public ACLExtension getACLExtension(final IMAPConfig imapConfig) {
+        if (instantiated.get() && MailAccount.DEFAULT_ID == imapConfig.getAccountId()) {
+            return configured;
+        }
+        return ACLExtensionAutoDetector.getACLExtension(imapConfig);
+    }
+
+    /**
+     * Gets the appropriate ACL extension for the IMAP server denoted by specified IMAP configuration.
+     *
+     * @param capabilities The capabilities map
+     * @param imapConfig The IMAP configuration providing needed access data.
+     * @return The appropriate ACL extension
+     */
+    public ACLExtension getACLExtension(final Map<String, String> capabilities, final IMAPConfig imapConfig) {
         if (!instantiated.get()) {
-            try {
-                return ACLExtensionAutoDetector.getACLExtension(imapConfig);
-            } catch (final IOException e) {
-                throw IMAPException.create(IMAPException.Code.IO_ERROR, e, e.getMessage());
-            }
+            return ACLExtensionAutoDetector.getACLExtension(capabilities, imapConfig);
         }
         return configured;
     }
@@ -120,7 +129,7 @@ public final class ACLExtensionFactory {
 
     /**
      * Only invoked if auto-detection is turned off.
-     * 
+     *
      * @param singleton The singleton instance of {@link ACLExtension}
      */
     void setACLExtensionInstance(final ACLExtension singleton) {
