@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -53,18 +53,17 @@ import static com.openexchange.imap.IMAPCommandsCollection.prepareStringArgument
 import java.util.Arrays;
 import java.util.Locale;
 import javax.mail.MessagingException;
-import com.openexchange.tools.Collections.SmartLongArray;
 import com.sun.mail.iap.Response;
 import com.sun.mail.imap.IMAPFolder;
 
 /**
  * {@link CopyIMAPCommand} - Copies messages from given folder to given destination folder just using their sequence numbers/UIDs
- * 
+ *
  * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
  */
 public final class CopyIMAPCommand extends AbstractIMAPCommand<long[]> {
 
-    private static final org.apache.commons.logging.Log LOG = org.apache.commons.logging.LogFactory.getLog(CopyIMAPCommand.class);
+    private static final org.apache.commons.logging.Log LOG = com.openexchange.log.Log.valueOf(org.apache.commons.logging.LogFactory.getLog(CopyIMAPCommand.class));
 
     private static final long[] DEFAULT_RETVAL = new long[0];
 
@@ -87,7 +86,7 @@ public final class CopyIMAPCommand extends AbstractIMAPCommand<long[]> {
     /**
      * Constructor using sequence numbers and performs a fast <code>COPY</code> command; meaning optional <i>COPYUID</i> response is
      * discarded.
-     * 
+     *
      * @param imapFolder - the IMAP folder
      * @param startSeqNum - the starting sequence number of the messages that shall be copied
      * @param endSeqNum - the ending sequence number of the messages that shall be copied
@@ -101,7 +100,7 @@ public final class CopyIMAPCommand extends AbstractIMAPCommand<long[]> {
     /**
      * Constructor using sequence numbers and performs a fast <code>COPY</code> command; meaning optional <i>COPYUID</i> response is
      * discarded.
-     * 
+     *
      * @param imapFolder - the IMAP folder
      * @param seqNums - the sequence numbers of the messages that shall be copied
      * @param destFolderName - the destination folder fullname
@@ -114,7 +113,7 @@ public final class CopyIMAPCommand extends AbstractIMAPCommand<long[]> {
 
     /**
      * Constructor using UIDs and consequently performs a <code>UID COPY</code> command
-     * 
+     *
      * @param imapFolder - the IMAP folder
      * @param uids - the UIDs of the messages that shall be copied
      * @param destFolderName - the destination folder fullname
@@ -132,19 +131,19 @@ public final class CopyIMAPCommand extends AbstractIMAPCommand<long[]> {
 
     private CopyIMAPCommand(final IMAPFolder imapFolder, final long[] nums, final String destFolderName, final boolean isSequential, final boolean fast, final boolean uid) throws MessagingException {
         super(imapFolder);
-        this.uids = nums == null ? DEFAULT_RETVAL : nums;
+        uids = nums == null ? DEFAULT_RETVAL : nums;
         this.uid = uid;
-        if (imapFolder.getMessageCount() == 0) {
+        if (imapFolder.getMessageCount() <= 0) {
             returnDefaultValue = true;
         } else {
-            returnDefaultValue = (this.uids.length == 0);
+            returnDefaultValue = (uids.length == 0);
         }
         this.fast = fast;
         this.destFolderName = prepareStringArgument(destFolderName);
-        length = this.uids.length;
-        args = length == 0 ? ARGS_EMPTY : (isSequential ? new String[] { new StringBuilder(64).append(this.uids[0]).append(':').append(
-            this.uids[length - 1]).toString() } : IMAPNumArgSplitter.splitUIDArg(
-            this.uids,
+        length = uids.length;
+        args = length == 0 ? ARGS_EMPTY : (isSequential ? new String[] { new StringBuilder(64).append(uids[0]).append(':').append(
+            uids[length - 1]).toString() } : IMAPNumArgSplitter.splitUIDArg(
+            uids,
             false,
             (uid ? LENGTH_WITH_UID : LENGTH) + destFolderName.length()));
         if (fast) {
@@ -159,22 +158,23 @@ public final class CopyIMAPCommand extends AbstractIMAPCommand<long[]> {
      * Constructor to copy all messages of given folder to given destination folder by performing a <code>COPY 1:*</code> command.
      * <p>
      * <b>Note</b>: Ensure that denoted folder is not empty.
-     * 
+     *
      * @param imapFolder - the IMAP folder
      * @param destFolderName - the destination folder
      * @throws MessagingException If a messaging error occurs
      */
     public CopyIMAPCommand(final IMAPFolder imapFolder, final String destFolderName) throws MessagingException {
         super(imapFolder);
-        if (imapFolder.getMessageCount() == 0) {
+        final int messageCount = imapFolder.getMessageCount();
+        if (messageCount <= 0) {
             returnDefaultValue = true;
         }
         fast = true;
         uid = false;
-        this.uids = DEFAULT_RETVAL;
+        uids = DEFAULT_RETVAL;
         this.destFolderName = prepareStringArgument(destFolderName);
         retval = DEFAULT_RETVAL;
-        args = ARGS_ALL;
+        args = 1 == messageCount ? new String[] { "1" } : ARGS_ALL;
         length = -1;
     }
 
@@ -213,133 +213,65 @@ public final class CopyIMAPCommand extends AbstractIMAPCommand<long[]> {
     private static final String COPYUID = "copyuid";
 
     @Override
-    protected void handleResponse(final Response response) throws MessagingException {
+    protected boolean handleResponse(final Response response) throws MessagingException {
+        if (!response.isOK()) {
+            return false;
+        }
         if (fast) {
-            return;
+            return true;
         }
         final String resp = response.toString().toLowerCase(Locale.ENGLISH);
         /*-
          * Parse response:
-         * 
+         *
          * OK [COPYUID 1184051486 10031:10523,10525:11020,11022:11027,11030:11047,11050:11051,11053:11558 1024:2544] Completed
-         * 
+         *
          * * 45 EXISTS..* 2 RECENT..A4 OK [COPYUID 1185853191 7,32 44:45] Completed
          */
         int pos = resp.indexOf(COPYUID);
-        if (pos == -1) {
+        if (pos < 0) {
             if (LOG.isWarnEnabled()) {
                 LOG.warn(new StringBuilder(128).append("Missing COPYUID response code: ").append(resp).toString());
             }
+            return true;
+        }
+        /*
+         * Found COPYUID...
+         */
+        final COPYUIDResponse copyuidResp = new COPYUIDResponse(LOG);
+        /*
+         * Find next starting ATOM in IMAP response
+         */
+        pos += COPYUID.length();
+        while (Character.isWhitespace(resp.charAt(pos))) {
+            pos++;
+        }
+        /*
+         * Split by ATOMs
+         */
+        final String[] sa = resp.substring(pos).split("\\s+");
+        if (sa.length >= 3) {
+            /*-
+             * Array contains atoms like:
+             *
+             * "1167880112", "11937", "11939]", "Completed"
+             */
+            copyuidResp.src = sa[1];
+            {
+                final String destStr = sa[2];
+                final int mlen = destStr.length() - 1;
+                if (']' == destStr.charAt(mlen)) {
+                    copyuidResp.dest = destStr.substring(0, mlen);
+                } else {
+                    copyuidResp.dest = destStr;
+                }
+            }
+            copyuidResp.fillResponse(uids, retval);
         } else {
-            final COPYUIDResponse copyuidResp = new COPYUIDResponse(LOG);
-            /*
-             * Find next starting ATOM in IMAP response
-             */
-            pos += COPYUID.length();
-            while (Character.isWhitespace(resp.charAt(pos))) {
-                pos++;
-            }
-            /*
-             * Split by ATOMs
-             */
-            final String[] sa = resp.substring(pos).split("\\s+");
-            if (sa.length >= 3) {
-                /*-
-                 * Array contains atoms like:
-                 * 
-                 * "1167880112", "11937", "11939]", "Completed"
-                 */
-                copyuidResp.src = sa[1];
-                {
-                    final String destStr = sa[2];
-                    final int mlen = destStr.length() - 1;
-                    if (']' == destStr.charAt(mlen)) {
-                        copyuidResp.dest = destStr.substring(0, mlen);
-                    } else {
-                        copyuidResp.dest = destStr;
-                    }
-                }
-                copyuidResp.fillResponse(uids, retval);
-            } else {
-                LOG.error(new StringBuilder(128).append("Invalid COPYUID response: ").append(resp).toString());
-            }
-            proceed = false;
-        }
-    }
-
-    private static final class COPYUIDResponse {
-
-        private final org.apache.commons.logging.Log logger;
-
-        String src;
-
-        String dest;
-
-        public COPYUIDResponse(final org.apache.commons.logging.Log logger) {
-            super();
-            this.logger = logger;
-        }
-
-        /**
-         * Fills given <code>retval</code> with UIDs from destination folder while keeping order of source UIDs in <code>uids</code>
-         * 
-         * @param uids The source UIDs
-         * @param retval The destination UIDs to fill
-         */
-        public void fillResponse(final long[] uids, final long[] retval) {
-            final long[] srcArr = toLongArray(src);
-            final long[] destArr = toLongArray(dest);
-            for (int in = 0; in < srcArr.length; in++) {
-                int index = 0;
-                /*
-                 * Determine index position in given UIDs...
-                 */
-                while ((index < uids.length) && (uids[index] != srcArr[in])) {
-                    index++;
-                }
-                /*
-                 * ... and apply copied UID to corresponding index position in return value
-                 */
-                try {
-                    retval[index] = destArr[in];
-                } catch (final ArrayIndexOutOfBoundsException e) {
-                    logger.error("A COPYUID's source UID could not be found in given source UIDs", e);
-                }
-            }
-        }
-
-        /**
-         * Turns a sequence of UIDs to a corresponding array of <code>long</code>.
-         * 
-         * <pre>
-         * 7,32,44:49
-         * </pre>
-         * 
-         * would be
-         * 
-         * <pre>
-         * [7,32,44,46,47,48,49]
-         * </pre>
-         * 
-         * @param uidSet The sequence of UIDs
-         * @return The corresponding array of <code>long</code>.
-         */
-        private static long[] toLongArray(final String uidSet) {
-            final SmartLongArray arr = new SmartLongArray();
-            final String[] sa = uidSet.split(" *, *");
-            Next: for (int i = 0; i < sa.length; i++) {
-                final int pos = sa[i].indexOf(':');
-                if (pos == -1) {
-                    arr.append(Long.parseLong(sa[i]));
-                    continue Next;
-                }
-                final long endUID = Long.parseLong(sa[i].substring(pos + 1));
-                for (long j = Long.parseLong(sa[i].substring(0, pos)); j <= endUID; j++) {
-                    arr.append(j);
-                }
-            }
-            return arr.toArray();
+            LOG.error(new StringBuilder(128).append("Invalid COPYUID response: ").append(resp).toString());
         }
+        proceed = false;
+        return true;
     }
 
     private static long[] startend2long(final int start, final int end) {
