@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -50,11 +50,17 @@
 package com.openexchange.imap.config;
 
 import java.nio.charset.Charset;
+import java.util.Arrays;
+import java.util.Collections;
+import java.util.HashSet;
 import java.util.Map;
+import java.util.Set;
 import java.util.concurrent.ConcurrentHashMap;
 import com.openexchange.config.ConfigurationService;
+import com.openexchange.imap.IMAPProtocol;
 import com.openexchange.imap.entity2acl.Entity2ACL;
 import com.openexchange.imap.services.IMAPServiceRegistry;
+import com.openexchange.mail.MailException;
 import com.openexchange.mail.api.AbstractProtocolProperties;
 import com.openexchange.mail.api.IMailProperties;
 import com.openexchange.mail.api.MailConfig.BoolCapVal;
@@ -64,18 +70,18 @@ import com.openexchange.spamhandler.SpamHandler;
 
 /**
  * {@link IMAPProperties}
- * 
+ *
  * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
  */
 public final class IMAPProperties extends AbstractProtocolProperties implements IIMAPProperties {
 
-    private static final org.apache.commons.logging.Log LOG = org.apache.commons.logging.LogFactory.getLog(IMAPProperties.class);
+    private static final org.apache.commons.logging.Log LOG = com.openexchange.log.Log.valueOf(org.apache.commons.logging.LogFactory.getLog(IMAPProperties.class));
 
     private static final IMAPProperties instance = new IMAPProperties();
 
     /**
      * Gets the singleton instance of {@link IMAPProperties}
-     * 
+     *
      * @return The singleton instance of {@link IMAPProperties}
      */
     public static IMAPProperties getInstance() {
@@ -94,6 +100,12 @@ public final class IMAPProperties extends AbstractProtocolProperties implements
 
     private boolean fastFetch;
 
+    private boolean notifyRecent;
+
+    private int notifyFrequencySeconds;
+
+    private String notifyFullNames;
+
     private BoolCapVal supportsACLs;
 
     private int imapTimeout;
@@ -110,19 +122,28 @@ public final class IMAPProperties extends AbstractProtocolProperties implements
 
     private int blockSize;
 
+    private int maxNumConnection;
+
     private final Map<String, Boolean> newACLExtMap;
 
     private String spamHandlerName;
 
     private boolean propagateClientIPAddress;
 
+    private boolean enableTls;
+
+    private Set<String> propagateHostNames;
+
     /**
      * Initializes a new {@link IMAPProperties}
      */
     private IMAPProperties() {
         super();
+        enableTls = true;
+        maxNumConnection = -1;
         newACLExtMap = new ConcurrentHashMap<String, Boolean>();
         mailProperties = MailProperties.getInstance();
+        propagateHostNames = Collections.emptySet();
     }
 
     @Override
@@ -131,6 +152,29 @@ public final class IMAPProperties extends AbstractProtocolProperties implements
         logBuilder.append("\nLoading global IMAP properties...\n");
 
         final ConfigurationService configuration = IMAPServiceRegistry.getService(ConfigurationService.class);
+        {
+            final String tmp = configuration.getProperty("com.openexchange.imap.notifyRecent", STR_FALSE).trim();
+            notifyRecent = Boolean.parseBoolean(tmp);
+            logBuilder.append("\tNotify Recent: ").append(notifyRecent).append('\n');
+        }
+
+        {
+            final String tmp = configuration.getProperty("com.openexchange.imap.notifyFrequencySeconds", "300").trim();
+            try {
+                notifyFrequencySeconds = Integer.parseInt(tmp);
+                logBuilder.append("\tNotify Frequency Seconds: ").append(notifyFrequencySeconds).append('\n');
+            } catch (final NumberFormatException e) {
+                notifyFrequencySeconds = 300;
+                logBuilder.append("\tNotify Frequency Seconds: Invalid value \"").append(tmp).append("\". Setting to fallback: ").append(
+                    notifyFrequencySeconds).append('\n');
+            }
+        }
+
+        {
+            final String tmp = configuration.getProperty("com.openexchange.imap.notifyFullNames", "INBOX").trim();
+            notifyFullNames = tmp;
+        }
+
         {
             final String imapSortStr = configuration.getProperty("com.openexchange.imap.imapSort", "application").trim();
             imapSort = "imap".equalsIgnoreCase(imapSortStr);
@@ -155,6 +199,23 @@ public final class IMAPProperties extends AbstractProtocolProperties implements
             logBuilder.append("\tPropagate Client IP Address: ").append(propagateClientIPAddress).append('\n');
         }
 
+        {
+            final String tmp = configuration.getProperty("com.openexchange.imap.enableTls", STR_TRUE).trim();
+            enableTls = Boolean.parseBoolean(tmp);
+            logBuilder.append("\tEnable TLS: ").append(enableTls).append('\n');
+        }
+
+        {
+            final String tmp = configuration.getProperty("com.openexchange.imap.propagateHostNames", "").trim();
+            if (tmp.length() > 0) {
+                propagateHostNames = Collections.unmodifiableSet(new HashSet<String>(Arrays.asList(tmp.split(" *, *"))));
+            } else {
+                propagateHostNames = Collections.emptySet();
+            }
+            logBuilder.append("\tPropagate Host Names: ").append(propagateHostNames.isEmpty() ? "<none>" : propagateHostNames.toString()).append(
+                '\n');
+        }
+
         {
             final String supportsACLsStr = configuration.getProperty("com.openexchange.imap.imapSupportsACL", STR_FALSE).trim();
             supportsACLs = BoolCapVal.parseBoolCapVal(supportsACLsStr);
@@ -241,6 +302,79 @@ public final class IMAPProperties extends AbstractProtocolProperties implements
             }
         }
 
+        {
+            String tmp = configuration.getProperty("com.openexchange.imap.maxNumExternalConnections");
+            if (null != tmp) {
+                tmp = tmp.trim();
+                if (0 == tmp.length()) {
+                    IMAPProtocol.getInstance().setOverallExternalMaxCount(-1);
+                    logBuilder.append("\tMax. Number of External Connections: ").append("No restrictions").append('\n');
+                } else if (tmp.indexOf(':') > 0) {
+                    // Expect a comma-separated list
+                    final String[] sa = tmp.split(" *, *");
+                    if (sa.length > 0) {
+                        try {
+                            final IMAPProtocol imapProtocol = IMAPProtocol.getInstance();
+                            imapProtocol.initExtMaxCountMap();
+                            final StringBuilder sb = new StringBuilder(128).append("\tMax. Number of External Connections: ");
+                            boolean first = true;
+                            for (final String desc : sa) {
+                                final int pos = desc.indexOf(':');
+                                if (pos > 0) {
+                                    try {
+                                        imapProtocol.putIfAbsent(desc.substring(0, pos), Integer.parseInt(desc.substring(pos + 1).trim()));
+                                        if (first) {
+                                            first = false;
+                                        } else {
+                                            sb.append(", ");
+                                        }
+                                        sb.append(desc);
+                                    } catch (final RuntimeException e) {
+                                        LOG.warn("Max. Number of External Connections: Invalid entry: " + desc, e);
+                                    }
+                                }
+                            }
+                            logBuilder.append(sb).append('\n');
+                        } catch (final NumberFormatException e) {
+                            IMAPProtocol.getInstance().setOverallExternalMaxCount(-1);
+                            logBuilder.append("\tMax. Number of External Connections: Invalid value \"").append(tmp).append(
+                                "\". Setting to fallback: No restrictions").append('\n');
+                        } catch (final MailException e) {
+                            IMAPProtocol.getInstance().setOverallExternalMaxCount(-1);
+                            logBuilder.append("\tMax. Number of External Connections: Invalid value \"").append(tmp).append(
+                                "\". Setting to fallback: No restrictions").append('\n');
+                        }
+                    }
+                } else {
+                    // Expect a single integer value
+                    try {
+                        IMAPProtocol.getInstance().setOverallExternalMaxCount(Integer.parseInt(tmp));
+                        logBuilder.append("\tMax. Number of External Connections: ").append(tmp).append(
+                            " (applied to all external IMAP accounts)").append('\n');
+                    } catch (final NumberFormatException e) {
+                        IMAPProtocol.getInstance().setOverallExternalMaxCount(-1);
+                        logBuilder.append("\tMax. Number of External Connections: Invalid value \"").append(tmp).append(
+                            "\". Setting to fallback: No restrictions").append('\n');
+                    }
+                }
+            }
+        }
+
+        {
+            final String tmp = configuration.getProperty("com.openexchange.imap.maxNumConnections", "-1").trim();
+            try {
+                maxNumConnection = Integer.parseInt(tmp);
+                logBuilder.append("\tMax. Number of connections: ").append(maxNumConnection).append('\n');
+                if (maxNumConnection > 0) {
+                    IMAPProtocol.getInstance().setMaxCount(maxNumConnection);
+                }
+            } catch (final NumberFormatException e) {
+                maxNumConnection = -1;
+                logBuilder.append("\tMax. Number of connections: Invalid value \"").append(tmp).append("\". Setting to fallback: ").append(
+                    maxNumConnection).append('\n');
+            }
+        }
+
         spamHandlerName = configuration.getProperty("com.openexchange.imap.spamHandler", SpamHandler.SPAM_HANDLER_FALLBACK).trim();
         logBuilder.append("\tSpam Handler: ").append(spamHandlerName).append('\n');
 
@@ -256,6 +390,8 @@ public final class IMAPProperties extends AbstractProtocolProperties implements
         imapSearch = false;
         fastFetch = true;
         propagateClientIPAddress = false;
+        enableTls = true;
+        propagateHostNames = Collections.emptySet();
         supportsACLs = null;
         imapTimeout = 0;
         imapConnectionTimeout = 0;
@@ -264,17 +400,41 @@ public final class IMAPProperties extends AbstractProtocolProperties implements
         imapAuthEnc = null;
         entity2AclImpl = null;
         blockSize = 0;
+        maxNumConnection = -1;
         spamHandlerName = null;
+        notifyRecent = false;
+        notifyFrequencySeconds = 300;
+        notifyFullNames = "INBOX";
     }
 
     public boolean isFastFetch() {
         return fastFetch;
     }
 
+    public boolean notifyRecent() {
+        return notifyRecent;
+    }
+
+    public int getNotifyFrequencySeconds() {
+        return notifyFrequencySeconds;
+    }
+
+    public String getNotifyFullNames() {
+        return notifyFullNames;
+    }
+
     public boolean isPropagateClientIPAddress() {
         return propagateClientIPAddress;
     }
 
+    public boolean isEnableTls() {
+        return enableTls;
+    }
+
+    public Set<String> getPropagateHostNames() {
+        return propagateHostNames;
+    }
+
     public String getImapAuthEnc() {
         return imapAuthEnc;
     }
@@ -309,7 +469,7 @@ public final class IMAPProperties extends AbstractProtocolProperties implements
 
     /**
      * Gets the {@link Entity2ACL}.
-     * 
+     *
      * @return The {@link Entity2ACL}
      */
     public String getEntity2AclImpl() {
@@ -320,13 +480,17 @@ public final class IMAPProperties extends AbstractProtocolProperties implements
         return blockSize;
     }
 
+    public int getMaxNumConnection() {
+        return maxNumConnection;
+    }
+
     public Map<String, Boolean> getNewACLExtMap() {
         return newACLExtMap;
     }
 
     /**
      * Gets the spam handler name.
-     * 
+     *
      * @return The spam handler name
      */
     public String getSpamHandlerName() {
@@ -353,10 +517,6 @@ public final class IMAPProperties extends AbstractProtocolProperties implements
         return mailProperties.getMailFetchLimit();
     }
 
-    public int getMaxNumOfConnections() {
-        return mailProperties.getMaxNumOfConnections();
-    }
-
     public int getWatcherFrequency() {
         return mailProperties.getWatcherFrequency();
     }
