@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -54,7 +54,7 @@ import com.openexchange.groupware.EnumComponent;
 
 /**
  * {@link Entity2ACLException} - Indicates errors related to entity2ACL mappings
- * 
+ *
  * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
  */
 public final class Entity2ACLException extends AbstractOXException {
@@ -115,7 +115,7 @@ public final class Entity2ACLException extends AbstractOXException {
 
         /**
          * Default constructor.
-         * 
+         *
          * @param message message.
          * @param category category.
          * @param detailNumber detail number.
@@ -123,7 +123,7 @@ public final class Entity2ACLException extends AbstractOXException {
         private Code(final String message, final Category category, final int detailNumber) {
             this.message = message;
             this.category = category;
-            this.number = detailNumber;
+            number = detailNumber;
         }
 
         /**
@@ -155,7 +155,7 @@ public final class Entity2ACLException extends AbstractOXException {
 
     /**
      * Initializes a new exception using the information provided by the cause.
-     * 
+     *
      * @param cause the cause of the exception.
      */
     public Entity2ACLException(final AbstractOXException cause) {
@@ -164,7 +164,7 @@ public final class Entity2ACLException extends AbstractOXException {
 
     /**
      * Initializes a new exception using the information provided by the code.
-     * 
+     *
      * @param code code for the exception.
      * @param messageArgs arguments that will be formatted into the message.
      */
@@ -174,7 +174,7 @@ public final class Entity2ACLException extends AbstractOXException {
 
     /**
      * Initializes a new exception using the information provided by the code.
-     * 
+     *
      * @param code code for the exception.
      * @param cause the cause of the exception.
      * @param messageArgs arguments that will be formatted into the message.
