@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -80,23 +80,34 @@ import com.openexchange.user.UserService;
  * </ul>
  * <p>
  * The complete implementation should be able to handle an ACL like this one:
- * 
+ *
  * <pre>
  * owner aceilrstwx anyone lr user=john w -user=mary r administrators aceilrstwx
  * </pre>
- * 
+ *
  * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
  */
 public class SUNMessagingServerEntity2ACL extends Entity2ACL {
 
-    private static final org.apache.commons.logging.Log LOG = org.apache.commons.logging.LogFactory.getLog(SUNMessagingServerEntity2ACL.class);
+    private static final org.apache.commons.logging.Log LOG = com.openexchange.log.Log.valueOf(org.apache.commons.logging.LogFactory.getLog(SUNMessagingServerEntity2ACL.class));
+
+    private static final SUNMessagingServerEntity2ACL INSTANCE = new SUNMessagingServerEntity2ACL();
+
+    /**
+     * Gets the instance.
+     *
+     * @return The instance
+     */
+    public static SUNMessagingServerEntity2ACL getInstance() {
+        return INSTANCE;
+    }
 
     private static final String ALIAS_ANYONE = "anyone";
 
     /**
      * Default constructor
      */
-    public SUNMessagingServerEntity2ACL() {
+    private SUNMessagingServerEntity2ACL() {
         super();
     }
 
