@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -69,7 +69,7 @@ import com.openexchange.mail.mime.HeaderCollection;
 
 /**
  * {@link ThreadSortMailMessage} - Extends {@link MailMessage} by capability to carry child messages.
- * 
+ *
  * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
  */
 public final class ThreadSortMailMessage extends MailMessage {
@@ -82,7 +82,7 @@ public final class ThreadSortMailMessage extends MailMessage {
 
     /**
      * Initializes a new {@link ThreadSortMailMessage}.
-     * 
+     *
      * @param delegatee
      */
     public ThreadSortMailMessage(final MailMessage delegatee) {
@@ -876,9 +876,39 @@ public final class ThreadSortMailMessage extends MailMessage {
         delegatee.writeTo(out);
     }
 
+    @Override
+    public boolean hasHeaders(String... names) {
+        return delegatee.hasHeaders(names);
+    }
+
+    @Override
+    public Date getReceivedDateDirect() {
+        return delegatee.getReceivedDateDirect();
+    }
+
+    @Override
+    public int getRecentCount() {
+        return delegatee.getRecentCount();
+    }
+
+    @Override
+    public boolean containsRecentCount() {
+        return delegatee.containsRecentCount();
+    }
+
+    @Override
+    public void removeRecentCount() {
+        delegatee.removeRecentCount();
+    }
+
+    @Override
+    public void setRecentCount(int recentCount) {
+        delegatee.setRecentCount(recentCount);
+    }
+
     /**
      * Adds specified child message.
-     * 
+     *
      * @param mailMessage The child message
      */
     public void addChildMessage(final ThreadSortMailMessage mailMessage) {
@@ -887,7 +917,7 @@ public final class ThreadSortMailMessage extends MailMessage {
 
     /**
      * Adds specified child messages.
-     * 
+     *
      * @param mailMessages The child messages
      */
     public void addChildMessages(final Collection<ThreadSortMailMessage> mailMessages) {
@@ -896,7 +926,7 @@ public final class ThreadSortMailMessage extends MailMessage {
 
     /**
      * Gets child messages.
-     * 
+     *
      * @return The child messages
      */
     public List<ThreadSortMailMessage> getChildMessages() {
@@ -905,7 +935,7 @@ public final class ThreadSortMailMessage extends MailMessage {
 
     /**
      * Gets the original message.
-     * 
+     *
      * @return The original message
      */
     public MailMessage getOriginalMessage() {
