@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -49,7 +49,8 @@
 
 package com.openexchange.imap.util;
 
-import gnu.trove.TLongHashSet;
+import gnu.trove.set.TLongSet;
+import gnu.trove.set.hash.TLongHashSet;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collection;
@@ -68,12 +69,12 @@ import com.sun.mail.imap.IMAPFolder;
 
 /**
  * {@link IMAPSessionStorage} - An IMAP storage held by a session.
- * 
+ *
  * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
  */
 final class IMAPSessionStorage {
 
-    private static final org.apache.commons.logging.Log LOG = org.apache.commons.logging.LogFactory.getLog(IMAPSessionStorage.class);
+    private static final org.apache.commons.logging.Log LOG = com.openexchange.log.Log.valueOf(org.apache.commons.logging.LogFactory.getLog(IMAPSessionStorage.class));
 
     private final Object lock;
 
@@ -81,7 +82,7 @@ final class IMAPSessionStorage {
 
     /**
      * Initializes a new {@link IMAPSessionStorage}.
-     * 
+     *
      * @param lock The lock object
      */
     IMAPSessionStorage(final Object lock) {
@@ -92,7 +93,7 @@ final class IMAPSessionStorage {
 
     /**
      * Checks if session storage contains entries for given folder.
-     * 
+     *
      * @param accountId The account ID
      * @param imapFolder The IMAP folder
      * @return <code>true</code> if session storage contains entries for given folder; otherwise <code>false</code>
@@ -103,7 +104,7 @@ final class IMAPSessionStorage {
 
     /**
      * Fills storage with data fetched from specified IMAP folder.
-     * 
+     *
      * @param accountId The account ID
      * @param imapFolder The IMAP folder
      * @throws MailException If a mail error occurs
@@ -129,7 +130,7 @@ final class IMAPSessionStorage {
 
     /**
      * Gets IMAP messages newly created, formerly deleted or of which flags have been changed since specified time stamp.
-     * 
+     *
      * @param accountId The account ID
      * @param imapFolder The IMAP folder of which messages are examined
      * @param mode The mode; either <code>1</code> for new-and-modified only, <code>2</code> for deleted only, or <code>3</code> for
@@ -153,9 +154,9 @@ final class IMAPSessionStorage {
                     dataMap.put(key, sessionData);
                 }
                 // Generate UID sets
-                final TLongHashSet actualUIDs = data2UIDSet(actualData);
-                final TLongHashSet dbUIDs = data2UIDSet(sessionData);
-                final TLongHashSet deleted;
+                final TLongSet actualUIDs = data2UIDSet(actualData);
+                final TLongSet dbUIDs = data2UIDSet(sessionData);
+                final TLongSet deleted;
                 if (((mode & 2) > 0)) {
                     deleted = new TLongHashSet(dbUIDs.size());
                     /*
@@ -183,13 +184,13 @@ final class IMAPSessionStorage {
                 } else {
                     deleted = new TLongHashSet(0);
                 }
-                final TLongHashSet newAndModified;
+                final TLongSet newAndModified;
                 if (((mode & 1) > 0)) {
                     newAndModified = new TLongHashSet(actualUIDs.size());
                     /*
                      * Detect new UIDs
                      */
-                    final TLongHashSet newUIDs = new TLongHashSet(actualUIDs);
+                    final TLongSet newUIDs = new TLongHashSet(actualUIDs);
                     newUIDs.removeAll(dbUIDs.toArray());
                     /*
                      * Insert new ones to session storage??? If yes, this routine's result are only yielded per call and thus are not
@@ -204,7 +205,7 @@ final class IMAPSessionStorage {
                     /*
                      * Detect UIDs of changed messages
                      */
-                    final TLongHashSet existing = new TLongHashSet(dbUIDs);
+                    final TLongSet existing = new TLongHashSet(dbUIDs);
                     existing.retainAll(actualUIDs.toArray());
                     if (!existing.isEmpty()) {
                         final Set<IMAPUpdateableData> changedSessionData = new HashSet<IMAPUpdateableData>(Arrays.asList(filterByUIDs(
@@ -215,7 +216,7 @@ final class IMAPSessionStorage {
                             /*
                              * Add UIDs of changed messages to appropriate set
                              */
-                            final TLongHashSet changedUIDs = data2UIDSet(changedSessionData);
+                            final TLongSet changedUIDs = data2UIDSet(changedSessionData);
                             newAndModified.addAll(changedUIDs.toArray());
                             /*
                              * Write changes to DB storage??? If yes, this routine's result are only yielded per call and thus are not
@@ -255,7 +256,7 @@ final class IMAPSessionStorage {
 
     /**
      * Removes specified deleted UIDs from session storage.
-     * 
+     *
      * @param deletedUIDs The set of deleted UIDs
      * @param accountId The account ID
      * @param fullName The IMAP folder's full name
@@ -271,7 +272,7 @@ final class IMAPSessionStorage {
 
     /**
      * Removes specified deleted UIDs from session storage.
-     * 
+     *
      * @param deletedUIDs The set of deleted UIDs
      * @param accountId The account ID
      * @param fullName The IMAP folder's full name
@@ -296,7 +297,7 @@ final class IMAPSessionStorage {
 
     /**
      * Removes specified deleted UIDs from session storage.
-     * 
+     *
      * @param accountId The account ID
      * @param fullName The IMAP folder's full name
      * @throws MailException If an error occurs while deleting UIDs
@@ -316,7 +317,7 @@ final class IMAPSessionStorage {
      * ############################ HELPER METHODS ############################
      */
 
-    private static TLongHashSet data2UIDSet(final IMAPUpdateableData[] updateableDatas) {
+    private static TLongSet data2UIDSet(final IMAPUpdateableData[] updateableDatas) {
         final TLongHashSet uids = new TLongHashSet(updateableDatas.length);
         for (int i = 0; i < updateableDatas.length; i++) {
             uids.add(updateableDatas[i].getUid());
@@ -324,7 +325,7 @@ final class IMAPSessionStorage {
         return uids;
     }
 
-    private static TLongHashSet data2UIDSet(final Collection<IMAPUpdateableData> updateableDatas) {
+    private static TLongSet data2UIDSet(final Collection<IMAPUpdateableData> updateableDatas) {
         final TLongHashSet uids = new TLongHashSet(updateableDatas.size());
         for (final IMAPUpdateableData updateableData : updateableDatas) {
             uids.add(updateableData.getUid());
@@ -332,7 +333,7 @@ final class IMAPSessionStorage {
         return uids;
     }
 
-    private static IMAPUpdateableData[] filterByUIDs(final TLongHashSet uids, final IMAPUpdateableData[] updateableDatas) {
+    private static IMAPUpdateableData[] filterByUIDs(final TLongSet uids, final IMAPUpdateableData[] updateableDatas) {
         final List<IMAPUpdateableData> tmp = new ArrayList<IMAPUpdateableData>(uids.size());
         for (int i = 0; i < updateableDatas.length; i++) {
             final IMAPUpdateableData updateableData = updateableDatas[i];
@@ -343,7 +344,7 @@ final class IMAPSessionStorage {
         return tmp.toArray(new IMAPUpdateableData[tmp.size()]);
     }
 
-    private static IMAPUpdateableData[] filterByUIDs(final TLongHashSet uids, final Collection<IMAPUpdateableData> updateableDatas) {
+    private static IMAPUpdateableData[] filterByUIDs(final TLongSet uids, final Collection<IMAPUpdateableData> updateableDatas) {
         final List<IMAPUpdateableData> tmp = new ArrayList<IMAPUpdateableData>(uids.size());
         for (final IMAPUpdateableData updateableData : updateableDatas) {
             if (uids.contains(updateableData.getUid())) {
