@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -56,13 +56,11 @@ import com.sun.mail.imap.IMAPFolder;
 
 /**
  * {@link IMAPSessionStorageAccess} - IMAP utility class for session storage.
- * 
+ *
  * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
  */
 public final class IMAPSessionStorageAccess {
 
-    private static final String SESSION_KEY = "imap.sessStorage";
-
     private static final boolean ENABLED = false;
 
     /**
@@ -72,23 +70,22 @@ public final class IMAPSessionStorageAccess {
         super();
     }
 
+    /**
+     * Checks if enabled.
+     * 
+     * @return <code>true</code> if enabled; otherwise <code>false</code>
+     */
+    public static boolean isEnabled() {
+        return ENABLED;
+    }
+
     private static IMAPSessionStorage ensureExistence(final Session session) {
-        IMAPSessionStorage param = (IMAPSessionStorage) session.getParameter(SESSION_KEY);
-        if (null == param) {
-            synchronized (session) {
-                param = (IMAPSessionStorage) session.getParameter(SESSION_KEY);
-                if (null == param) {
-                    param = new IMAPSessionStorage(session);
-                    session.setParameter(SESSION_KEY, param);
-                }
-            }
-        }
-        return param;
+        return new IMAPSessionStorage(session);
     }
 
     /**
      * Checks if session storage contains entries for given folder.
-     * 
+     *
      * @param accountId The account ID
      * @param imapFolder The IMAP folder
      * @param session The session providing user data
@@ -103,7 +100,7 @@ public final class IMAPSessionStorageAccess {
 
     /**
      * Fills session storage with data fetched from specified IMAP folder.
-     * 
+     *
      * @param accountId The account ID
      * @param imapFolder The IMAP folder
      * @param session The session providing user data
@@ -118,7 +115,7 @@ public final class IMAPSessionStorageAccess {
 
     /**
      * Gets IMAP messages newly created, formerly deleted or of which flags have been changed since specified time stamp.
-     * 
+     *
      * @param accountId The account ID
      * @param imapFolder The IMAP folder of which messages are examined
      * @param session The session providing user data
@@ -136,7 +133,7 @@ public final class IMAPSessionStorageAccess {
 
     /**
      * Removes specified deleted UIDs from session storage.
-     * 
+     *
      * @param deletedUIDs The set of deleted UIDs
      * @param accountId The account ID
      * @param session The session
@@ -152,7 +149,7 @@ public final class IMAPSessionStorageAccess {
 
     /**
      * Removes specified deleted UIDs from session storage.
-     * 
+     *
      * @param deletedUIDs The set of deleted UIDs
      * @param accountId The account ID
      * @param session The session
@@ -168,7 +165,7 @@ public final class IMAPSessionStorageAccess {
 
     /**
      * Removes specified deleted UIDs from session storage.
-     * 
+     *
      * @param accountId The account ID
      * @param session The session
      * @param fullName The IMAP folder's full name
